@@ -724,8 +724,8 @@ where
 	/// ```rust,should_panic
 	/// use bitvec::prelude::*;
 	///
-	/// let bits = BitSlice::<LocalBits, usize>::empty_mut();
-	/// bits.set(0, false);
+	/// let bits = bits![mut 0];
+	/// bits.set(1, false);
 	/// ```
 	#[inline]
 	pub fn set(&mut self, index: usize, value: bool) {
@@ -811,9 +811,9 @@ where
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bits = 0xFDu8.view_bits::<Msb0>();
-	/// assert!(bits[.. 4].all());
-	/// assert!(!bits[4 ..].all());
+	/// let bits = bits![1, 1, 0, 1];
+	/// assert!(bits[.. 2].all());
+	/// assert!(!bits[2 ..].all());
 	/// ```
 	#[inline]
 	pub fn all(&self) -> bool {
@@ -868,9 +868,9 @@ where
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bits = 0x40u8.view_bits::<Msb0>();
-	/// assert!(bits[.. 4].any());
-	/// assert!(!bits[4 ..].any());
+	/// let bits = bits![0, 1, 0, 0];
+	/// assert!(bits[.. 2].any());
+	/// assert!(!bits[2 ..].any());
 	/// ```
 	#[inline]
 	pub fn any(&self) -> bool {
@@ -913,9 +913,9 @@ where
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bits = 0xFDu8.view_bits::<Msb0>();
-	/// assert!(!bits[.. 4].not_all());
-	/// assert!(bits[4 ..].not_all());
+	/// let bits = bits![1, 1, 0, 1];
+	/// assert!(!bits[.. 2].not_all());
+	/// assert!(bits[2 ..].not_all());
 	/// ```
 	#[inline]
 	pub fn not_all(&self) -> bool {
@@ -946,9 +946,9 @@ where
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bits = 0x40u8.view_bits::<Msb0>();
-	/// assert!(!bits[.. 4].not_any());
-	/// assert!(bits[4 ..].not_any());
+	/// let bits = bits![0, 1, 0, 0];
+	/// assert!(!bits[.. 2].not_any());
+	/// assert!(bits[2 ..].not_any());
 	/// ```
 	#[inline]
 	pub fn not_any(&self) -> bool {
@@ -984,10 +984,10 @@ where
 	/// use bitvec::prelude::*;
 	///
 	/// let data = 0b111_000_10u8;
-	/// let bits = data.view_bits::<Msb0>();
+	/// let bits = bits![1, 1, 0, 0, 1, 0];
 	///
-	/// assert!(!bits[.. 3].some());
-	/// assert!(!bits[3 .. 6].some());
+	/// assert!(!bits[.. 2].some());
+	/// assert!(!bits[2 .. 4].some());
 	/// assert!(bits.some());
 	/// ```
 	///
@@ -1015,11 +1015,9 @@ where
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let data = 0xF0u8;
-	/// let bits = data.view_bits::<Msb0>();
-	///
-	/// assert_eq!(bits[.. 4].count_ones(), 4);
-	/// assert_eq!(bits[4 ..].count_ones(), 0);
+	/// let bits = bits![1, 1, 0, 0];
+	/// assert_eq!(bits[.. 2].count_ones(), 2);
+	/// assert_eq!(bits[2 ..].count_ones(), 0);
 	/// ```
 	#[inline]
 	pub fn count_ones(&self) -> usize {
@@ -1063,11 +1061,9 @@ where
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let data = 0xF0u8;
-	/// let bits = data.view_bits::<Msb0>();
-	///
-	/// assert_eq!(bits[.. 4].count_zeros(), 0);
-	/// assert_eq!(bits[4 ..].count_zeros(), 4);
+	/// let bits = bits![1, 1, 0, 0];
+	/// assert_eq!(bits[.. 2].count_zeros(), 0);
+	/// assert_eq!(bits[2 ..].count_zeros(), 2);
 	/// ```
 	#[inline]
 	pub fn count_zeros(&self) -> usize {
@@ -1213,6 +1209,19 @@ where
 	///
 	/// A view of the entire memory region this slice covers, including the edge
 	/// elements.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::prelude::*;
+	///
+	/// let data = 0x3Cu8;
+	/// let bits = &data.view_bits::<LocalBits>()[2 .. 6];
+	///
+	/// assert!(bits.all());
+	/// assert_eq!(bits.len(), 4);
+	/// assert_eq!(bits.as_slice(), &[0x3Cu8]);
+	/// ```
 	#[inline]
 	#[cfg(not(tarpaulin_include))]
 	pub fn as_slice(&self) -> &[T] {
@@ -1374,8 +1383,7 @@ where
 	/// use bitvec::prelude::*;
 	/// type Bs<T> = BitSlice<LocalBits, T>;
 	///
-	/// let mut data = [0u8; 3];
-	/// let bits = data.view_bits_mut::<LocalBits>();
+	/// let bits = bits![mut LocalBits, u8; 0; 24];
 	/// let (a, b): (
 	///   &mut Bs<<u8 as BitStore>::Alias>,
 	///   &mut Bs<<u8 as BitStore>::Alias>,
@@ -1492,6 +1500,7 @@ where
 	///
 	/// [`.domain`]: #method.domain
 	#[inline]
+	#[cfg(not(tarpaulin_include))]
 	pub fn domain_mut(&mut self) -> DomainMut<T> {
 		DomainMut::new(self)
 	}
@@ -1766,6 +1775,100 @@ where
 		}
 	}
 
+	/// Produces the absolute offset in bits between two slice heads.
+	///
+	/// While this method is sound for any two arbitrary bit slices, the answer
+	/// it produces is meaningful *only* when one argument is a strict subslice
+	/// of the other. If the two slices are created from different buffers
+	/// entirely, a comparison is undefined; if the two slices are disjoint
+	/// regions of the same buffer, then the semantically correct distance is
+	/// between the tail of the lower and the head of the upper, which this
+	/// does not measure.
+	///
+	/// # Visual Description
+	///
+	/// Consider the following sequence of bits:
+	///
+	/// ```text
+	/// [ 0 1 2 3 4 5 6 7 8 9 a b ]
+	///   |       ^^^^^^^       |
+	///   ^^^^^^^^^^^^^^^^^^^^^^^
+	/// ```
+	///
+	/// It does not matter whether there are bits between the tail of the
+	/// smaller and the larger slices. The offset is computed from the bit
+	/// distance between the two heads.
+	///
+	/// # Behavior
+	///
+	/// This function computes the *semantic* distance between the heads, rather
+	/// than the *electrical. It does not take into account the `BitOrder`
+	/// implementation of the slice. See the [`::electrical_distance`] method
+	/// for that comparison.
+	///
+	/// # Safety and Soundness
+	///
+	/// One of `self` or `other` must contain the other for this comparison to
+	/// be meaningful.
+	///
+	/// # Parameters
+	///
+	/// - `&self`
+	/// - `other`: Another bit slice. This must be either a strict subregion or
+	///   a strict superregion of `self`.
+	///
+	/// # Returns
+	///
+	/// The distance in (semantic) bits betwen the heads of each region. The
+	/// value is positive when `other` is higher in the address space than
+	/// `self`, and negative when `other` is lower in the address space than
+	/// `self`.
+	///
+	/// [`::electrical_distance]`: #method.electrical_comparison
+	pub fn offset_from(&self, other: &Self) -> isize {
+		let (elts, bits) = unsafe { self.bitptr().ptr_diff(other.bitptr()) };
+		elts.saturating_mul(T::Mem::BITS as isize)
+			.saturating_add(bits as isize)
+	}
+
+	/// Computes the electrical distance between the heads of two slices.
+	///
+	/// This method uses the slices’ `BitOrder` implementation to compute the
+	/// bit position of their heads, then computes the shift distance, in bits,
+	/// between them.
+	///
+	/// This computation presumes that the bits are counted in the same
+	/// direction as are bytes in the abstract memory map.
+	///
+	/// # Parameters
+	///
+	/// - `&self`
+	/// - `other`: Another bit slice. This must be either a strict subregion or
+	///   a strict superregion of `self`.
+	///
+	/// # Returns
+	///
+	/// The electrical bit distance between the heads of `self` and `other`.
+	pub fn electrical_distance(&self, other: &Self) -> isize {
+		let this = self.bitptr();
+		let that = other.bitptr();
+		let (elts, bits) = unsafe {
+			let this = BitPtr::new_unchecked(
+				this.pointer(),
+				BitIdx::new_unchecked(this.head().position::<O>().value()),
+				1,
+			);
+			let that = BitPtr::new_unchecked(
+				that.pointer(),
+				BitIdx::new_unchecked(that.head().position::<O>().value()),
+				1,
+			);
+			this.ptr_diff(that)
+		};
+		elts.saturating_mul(T::Mem::BITS as isize)
+			.saturating_add(bits as isize)
+	}
+
 	/// Marks an immutable slice as referring to aliased memory region.
 	#[inline(always)]
 	#[cfg(not(tarpaulin_include))]
@@ -1797,6 +1900,7 @@ where
 
 	/// Type-cast the slice reference to its pointer structure.
 	#[inline]
+	#[cfg(not(tarpaulin_include))]
 	pub(crate) fn bitptr(&self) -> BitPtr<T> {
 		BitPtr::from_bitslice_ptr(self.as_ptr())
 	}
@@ -1805,6 +1909,7 @@ where
 	///
 	/// This is restricted so that it can only be used within the crate.
 	/// Construction of a `BitSlice` over externally-aliased memory is unsound.
+	#[cfg(not(tarpaulin_include))]
 	pub(crate) unsafe fn from_aliased_slice_unchecked(
 		slice: &[T::Alias],
 	) -> &BitSlice<O, T::Alias> {
